package juniq;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;
import java.util.*;
import java.util.concurrent.*;
import java.security.*;
import javax.swing.table.*;

public class EmptyFolders implements Serializable
{
    public static final long serialVersionUID = 1001L;

    JFrame frame;
    JTextArea textarea;

    JMenuBar menuBar = new JMenuBar();

    JMenu     opMenu = new JMenu("Operations");
    JMenuItem addItem = new JMenuItem("Add path...");
    JMenuItem genItem = new JMenuItem("Generate delete script...");
    JMenuItem quitItem = new JMenuItem("Quit");

    ArrayList<File>    emptyDirs = new ArrayList<File>();

    AbstractTableModel tableModel = new EmptyFoldersTableModel();
    TableSorter tableSorter = new TableSorter(tableModel); //, table.getTableHeader());
    JTable    table = new JTable(tableSorter);

    JFileChooser  pathChooser = new JFileChooser();

    JLabel  locatedFilesLabel = new JLabel("Located: 0 Directories");
    JLabel  statusLabel = new JLabel("idle");

    BlockingQueue<File>  rootQueue = new LinkedBlockingQueue<File>();

    EmptyFoldersGenerate generate = new EmptyFoldersGenerate(this);
    DirectorySearcherThread dst;

    public EmptyFolders()
    {
	frame = new JFrame("Empty Folders, by Edwin Olson (eolson@mit.edu)");
	frame.getContentPane().setLayout(new BorderLayout());

	menuBar.add(opMenu);
	opMenu.add(addItem);
	opMenu.add(genItem);
	opMenu.add(quitItem);

	quitItem.addActionListener(new ActionListener() 
	    { public void actionPerformed(ActionEvent e) { fileQuit(); }});
	addItem.addActionListener(new ActionListener() 
	    { public void actionPerformed(ActionEvent e) { opAdd(); }});
	genItem.addActionListener(new ActionListener() 
	    { public void actionPerformed(ActionEvent e) { opGenerate(); }});
	
	frame.setJMenuBar(menuBar);

	dst = new DirectorySearcherThread();
	dst.start();

	StatusThread st = new StatusThread();
	st.start();

	table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

	/////////////////////////////////////////////////
	frame.getContentPane().add(new JScrollPane(table), BorderLayout.CENTER);

	Box hbox1 = new Box(BoxLayout.Y_AXIS);
	hbox1.add(locatedFilesLabel);
	hbox1.add(statusLabel);

	frame.getContentPane().add(hbox1, BorderLayout.SOUTH);

	frame.setSize(800,600);
	frame.setVisible(true);
    }

    void updateStatus()
    {
	if (dst.running)
	    statusLabel.setText("Searching...");
	else
	    statusLabel.setText("Idle");
	locatedFilesLabel.setText(String.format("Located: %d directories", emptyDirs.size()));
    }


    public static void main(String[] args)
    {
	EmptyFolders app = new EmptyFolders();

	for (String s:args)
	    {
		try {
		    File f = new File(s);
		    if (f.isDirectory())
			app.rootQueue.put(new File(s));
		} catch (InterruptedException ex) {
		}
	    }
    }

    static boolean isSymbolicLink(File f)
    {
	try {
	    return (!f.getAbsolutePath().equals(f.getCanonicalPath()));
	} catch (IOException ex) {
	    System.out.println("Couldn't determine whether "+f+" is sym link! "+ex);
	    System.exit(0);
	    return false;
	}
    }

    void fileQuit()
    {
	System.exit(0);
    }

    void opAdd()
    {
	pathChooser.setMultiSelectionEnabled(true);
	pathChooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);

	int ret = pathChooser.showOpenDialog(frame);
	if (ret != JFileChooser.APPROVE_OPTION)
	    return;

	File[] files = pathChooser.getSelectedFiles();

	for (File f : files)
	    {
		try {
		    rootQueue.put(f);
		} catch (InterruptedException ex) {
		}
	    }
    }

    void opGenerate()
    {
	generate.setVisible(true);
    }

    class DirectorySearcherThread extends Thread
    {
	public boolean running = false;

	public DirectorySearcherThread()
	{
	    setDaemon(true);
	}

	public void run()
	{
	    while (true)
		{
		    try {
			File f = rootQueue.take();
			running=true;
			recurse(f);
			running=false;
		    } catch (InterruptedException ex) {
		    }
		}
	}

	public void recurse(File file)
	{
	    // ignore symlinks
	    if (isSymbolicLink(file))
		{
		    //		    symLinks.add(file);
		    System.out.println("Detected symlink: "+file);
		    return;
		}

	    // descend recursively.
	    if (file.isDirectory())
		{
		    File[] files = file.listFiles();
		    if (files==null || files.length==0) {
			emptyDirs.add(file);
			int idx = emptyDirs.size() - 1;
			tableModel.fireTableRowsInserted(idx, idx);
			//			System.out.println("adding");
								      
			return;
		    }

		    for (File thisfile : files)
			{
			    recurse(thisfile);
			}
		    return;
		}
	    
	    if (!file.isFile())
		{
		    System.out.println("Detected non file, non directory: " + file);
		    //		    nonFiles.add(file);
		    return;
		}
	}
    }

    class EmptyFoldersTableModel extends AbstractTableModel
    {
	public static final long serialVersionUID = 1001;

	public synchronized int getRowCount()
	{
	    return emptyDirs.size();
	}

	public int getColumnCount()
	{
	    return 1;
	}

	public Class getColumnClass(int column)
	{
	    if (column==0)
		return String.class;
	    return null;
	}

	public Object getValueAt(int row, int column)
	{
	    File f = emptyDirs.get(row);

	    if (column==0)
		return f.getAbsolutePath();

	    return "";
	}

	public String getColumnName(int col)
	{
	    switch (col)
		{
		case 0:
		    return "Directory";
		}
	    return "?";
	}
    }

   class StatusThread extends Thread
    {
	public StatusThread()
	{
	    setDaemon(true);
	}

	public void run()
	{
	    while (true)
		{
		    updateStatus();

		    try {
			Thread.sleep(200);
		    } catch (InterruptedException ex) {
		    }
		}
	}
    }

}
